/*
 * Copyright 2009 UbiCollab.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ubicollab.rdp.tan.model;

import java.util.ArrayList;
import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.*;


public class ServiceList {
	
	String fileName;
	ArrayList<Service> list = null;
	public ServiceList()
	{
		updateServiceList();
	}
	
	private void updateServiceList()
	{
		list = new ArrayList<Service>();
		addDefaultServices();
		//Check which drive on the device is writable
		int storageWritable =0;
		for(int j=0;j<2;j++)
		{
			File checkIfWritable = new File(java.io.File.listRoots()[j].getAbsolutePath());
			if(checkIfWritable.canWrite())
				storageWritable = j;
		}
		String drive = java.io.File.listRoots()[storageWritable].getAbsolutePath();
		System.out.println("Device in use: "+drive);
		fileName = drive + Settings.STORAGE_ROOT + System.getProperty("file.separator") + Settings.SERVICE_FILE;
		
		//Check if a customized copy of the Service List is locally available, if not it fill it
		//the default services
		File checkIfExists = new File(fileName);
		if(!checkIfExists.exists())
		{
			addDefaultServices();
			try {
				Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
				writeToFile(list, fileName);
			} catch (Exception e) {
				System.out.println("Cannot write services.xml");
				e.printStackTrace();
			}
		}
		
		File path = new File(drive+Settings.STORAGE_ROOT);
		boolean directoryExist = true;
		if ( !path.exists() ) {
			directoryExist = path.mkdirs();
		}
		if ( !directoryExist ) {
			list = new ArrayList<Service>();
		}
		else {
			try {
				Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
				list = (ArrayList<Service>) readFromFile(fileName);
			} catch (Exception e) {
				list = new ArrayList<Service>();
				e.printStackTrace();
			}
		}
	}


	
	private static void writeToFile(ArrayList sl, String filename) throws Exception {
        XMLEncoder encoder =
           new XMLEncoder(
              new BufferedOutputStream(
                new FileOutputStream(filename)));
        encoder.writeObject(sl);
        encoder.close();
    }
	
	private static ArrayList readFromFile(String filename) throws Exception {
        File f = new File(filename);
        ArrayList readObject;
        if ( f.exists() ) {
			XMLDecoder decoder = new XMLDecoder(new BufferedInputStream( new FileInputStream(filename)));
			readObject = (ArrayList) decoder.readObject();
			decoder.close();
		}
        else {
        	readObject = new ArrayList<Service>();
        }
        return readObject;
    }
	
	private void addDefaultServices()
	{
		list.add(new Service(1111,"UbiDummy","Proxy Service","UC Room","Simone","A test proxyservice","http://folk.ntnu.no/simonem/ubi_dummy_proxy.jar","http://folk.ntnu.no/simonem/ubi_dummy_proxy.html"));
		list.add(new Service(1234,"UbiNabaztag","Proxy Service","UC Room","Alfredo","A test proxyservice","http://folk.ntnu.no/simonem/ubi_nazbatag_proxy.jar","http://folk.ntnu.no/simonem/ubi_nazbatag_proxy.html"));
		list.add(new Service(1212,"UbiX10","Proxy Service","UC Room","Simone","A test proxyservice","http://folk.ntnu.no/simonem/ubi_X10_proxy.jar","http://folk.ntnu.no/simonem/ubi_X10_proxy.html"));
		list.add(new Service(2323,"OpenWallImagePresenter","UCProxy","UC Room","Simone","A proxy for the open wall image presenter","http://folk.ntnu.no/simonem/bundles/no.ubicollab.proxy.openwallimgpresenter.jar","http://folk.ntnu.no/simonem/bundles/no.ubicollab.proxy.openwallimgpresenter.jar"));
		list.add(new Service(3333,"ImagePresenter","UCProxy","UC Room","Simone","A proxy for the image presenter","http://folk.ntnu.no/simonem/bundles/no.ubicollab.proxy.imagepresenter.jar","http://folk.ntnu.no/simonem/bundles/no.ubicollab.proxy.imagepresenter.jar"));
	}	
	
	public ArrayList<Service> getServiceList()
	{
		return list;
	}

	public Service findService(int id)
	{
		for(Service serv : list)
		{
			if(serv.getId()==id)
				return serv;
			}
	return null;
	}
	
	public String findServiceName(int id)
	{
		for(Service serv : list)
		{
			if(serv.getId()==id)
				return serv.getName();
			}
	return null;
	}
	public String toXML() {
		Service s;
		String xml = Settings.XML_HEADER;
		xml += "<Servicelist>";
		for (int i=0;i<list.size();i++) {
			s = list.get(i);
			xml += s.toXML(); 
		}
		xml += "</Servicelist>";
		return xml;
	}
	
	
}

