/*
 * Copyright 2009 UbiCollab.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ubicollab.service.imgpresenter.ui;


import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ercp.swt.mobile.*;
import org.ubicollab.service.imgpresenter.model.Controller;
import org.ubicollab.service.imgpresenter.model.ImagePresenter;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;


/**
 * @author  Simone
 */
public class MainView extends ViewPart implements PaintListener, KeyListener, SelectionListener{
	
    Canvas  canvas;
	String  drawCase="//res/adv.jpg";
	int     counter;
	boolean eraseBgd;
	Command commandExit;
	Command commandNext;
    private FocusListener superShellListener = null;
	Color col1;
	Color col2;
	public static final String ID = "MainView";
	/**
	 * @uml.property  name="controller"
	 * @uml.associationEnd  
	 */
	Controller controller=null;
	
	public MainView() {	
	}

	/**
	 * Create contents of the view part.
	 */
	public void createPartControl(Composite parent) {
		controller=Controller.getInstance();
		controller.registerView(this);
		Controller.setDisplay(parent.getDisplay());
	    canvas = new Canvas(parent, SWT.NO_BACKGROUND);
	   
		    
		//Listener on canvas
	    canvas.addMouseListener(new MouseListener() {
	    	public void mouseDoubleClick(MouseEvent arg0) {
	    	}
	    	public void mouseDown(MouseEvent arg0) {
	    	}
	    	public void mouseUp(MouseEvent arg0) {
	    	}
	    });
	        
	    //Define the screen painting area
	    Rectangle area = parent.getClientArea();
	    canvas.setBounds(area);
	    canvas.addPaintListener(this);
	    canvas.addKeyListener(this);	
		eraseBgd = true;

		// create Commands			
		commandNext = new Command(canvas, Command.SELECT,3);
		commandNext.setText("command NEXT"); //$NON-NLS-1$
		commandNext.addSelectionListener(this);
		commandNext.setDefaultCommand();

		commandExit = new Command(canvas, Command.EXIT,1);
		commandExit.setText("label"); //$NON-NLS-1$
		commandExit.setLongLabel("longlabel"); //$NON-NLS-1$
		commandExit.addSelectionListener(this);
		
	    canvas.setFocus();
	    
		//Bring up this cavas image if existed 
		superShellListener = new FocusListener() {
			public void focusGained(FocusEvent arg0) {
				if(canvas != null && !canvas.isDisposed()) {
					eraseBgd = true;
					//Trigger a redraw event
					canvas.redraw();
					canvas.forceFocus();
				}
			}

			public void focusLost(FocusEvent arg0) {
			}			
		};
	}

	 //WS METHODS
    public void drawTarget(String url)
    {
    	drawCase = url;
    	eraseBgd = true;
		canvas.redraw();
    }
    public void showAdvertaisment()
    {
    	drawCase = "//res/adv.jpg";
    	eraseBgd = true;
    	canvas.redraw();
    }
    
	/**
	 * Drawing operations
	 */
	public void paintControl(PaintEvent e) {
		if (eraseBgd) {
			e.gc.fillRectangle(canvas.getClientArea());
			eraseBgd = false;
		}    
			drawImage(e, drawCase); 
	}
	
	/**
	 * Press any key to go through the different screens
	 */
	public void keyPressed(KeyEvent e) {
	}
	
	public void keyReleased(KeyEvent e) {
	}
	
	/**
	 * display an image 
	 * 
	 * @param e	Paintevent
	 * @param name filename
	 */
	private void drawImage(PaintEvent e, String name) {
		String def = "//res/adv.jpg";
		URL url = null;
		Image img = null;
		if(name.equals(def))
		{
			img = new Image(canvas.getDisplay(), getClass().getResourceAsStream(name));
		}
		else
		{
		try {
			url = new URL(name);
			img = new Image(canvas.getDisplay(), url.openStream());
		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		}
		Rectangle imageBnds  = img.getBounds();
		Rectangle canvasBnds = canvas.getBounds();
		int x = (canvasBnds.width  - imageBnds.width ) / 2;
		int y = (canvasBnds.height - imageBnds.height) / 2;
		GC gc = new GC(canvas, SWT.NONE);
		gc.fillRectangle(canvas.getClientArea());
		gc.drawImage(img, x, y);
		img.dispose();
		
		/*
		String url = "http://folk.ntnu.no/simonem/bundles/bundle_support/img.jpg";
		URL uri = null;
		try {
			uri = new URL(url);
		} catch (MalformedURLException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}
		try {
			Image img2 = new Image(canvas.getDisplay(),uri.openStream());
			gc.drawImage(img2, x, y);
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}*/	
	}
		
	public void widgetSelected(SelectionEvent e) {
		if (e.widget==commandExit) {
		    counter = 0;
			quit();
		}
	}
	
	public void quit() {
	    if (col1 != null) {
	        col1.dispose();
	    }
	    if (col2 != null) {
	        col2.dispose();
	    }
	    if(canvas.getShell() != null && !canvas.getShell().isDisposed())
		{	
	    	canvas.getParent().getShell().removeFocusListener(superShellListener);
		}   
	    canvas.getParent().redraw();
	    dispose();
	}
	public void widgetDefaultSelected(SelectionEvent e) {
 	}
	public void setFocus() {
	}
}
