/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.image;

import java.awt.image.RGBImageFilter;

/**
 * An image filter to map from rgb to r (led color) only.
 * Current implementation gives same weight to all intensities,
 * hence white will give full led intensity. 
 * If used in combination (before) the WallImageFilter,
 * a wall-like image may be produced.
 * @author Hallvard Trtteberg
 *
 */
public class RGB2LedImageFilter extends RGBImageFilter {

	private int redWeight = 255;
	private int greenWeight = 255;
	private int blueWeight = 255;
	
	public RGB2LedImageFilter(int redWeight, int greenWeight, int blueWeight) {
		canFilterIndexColorModel = true;
		this.redWeight = redWeight;
		this.greenWeight = greenWeight;
		this.blueWeight = blueWeight;
	}

	public RGB2LedImageFilter() {
		this(255, 255, 255);
	}

	public int filterRGB(int x, int y, int rgb) {
		int alpha = rgb & 0xff000000; 
		int red =   (rgb >> 16) & 0xff; 
		int green = (rgb >>  8) & 0xff; 
		int blue  = (rgb >>  0) & 0xff;
		int ledRgb = (red * redWeight + green * greenWeight + blue * blueWeight) / (redWeight + greenWeight + blueWeight);
		return alpha | (ledRgb << 16);
	}
}
