/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.model;

import java.util.ArrayList;
import java.util.List;

/**
 * An imcomplete (abstract) implementation of WallModel,
 * implements support for a List of WallListeners and
 * a method for notifying them of a change.
 * 
 * @author Hallvard Trtteberg
 *
 */
public abstract class AbstractWallModel implements WallModel {

	private List<WallListener> listeners = new ArrayList<WallListener>();
	
	public void addWallListener(WallListener listener) {
		listeners.add(listener);
	}
	
	public void removeWallListener(WallListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Notifies the set of listeners of changes to a rectangle of leds.
	 * There is no guarantee that the set of notifications is optimal,
	 * e.g. changes to a rectangle may given in smaller pieces,
	 * even individual leds where both width and height is 1.
	 * @param x the x-coordinate
	 * @param y the y-coordinate
	 * @param width the width of the changed area
	 * @param height the height of the changed area
	 * @param value the new value
	 */
	protected void notifyListeners(int x, int y, int width, int height, byte value) {
		for (WallListener listener: listeners) {
			if (width == 1 && height == 1) {
				listener.ledChanged(x, y, value, this);
			}
			listener.ledsChanged(x, y, width, height, this);
		}
	}

	public AbstractWallModel() {
	}
	
	/**
	 * A default implementation of setLeds, which uses the setLed method for setting each led.
	 */
	public void setLeds(int x0, int y0, int dx, int dy, byte value) {
		for (int x = x0; x < x0 + dx; x++) {
			for (int y = y0; y < y0 + dy; y++) {
				setLed(x, y, value);
			}
		}
		notifyListeners(x0, y0, dx, dy, value);
	}
}
