/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.model;

/**
 * Interface for listening to changes to the wall, or rather, to a WallModel.
 * @author Hallvard Trtteberg
 *
 */
public interface WallListener {
	
	/**
	 * This method is called for each change to a single led, at position x,y, to value 0.
	 * @param x the x-coordinate
	 * @param y the y-coordinate
	 * @param value the new value
	 * @param model the WallModel that changed
	 */
	public void ledChanged(int x, int y, byte value, WallModel model);

	/**
	 * This method reports changes to a rectangle of leds, at position x,y, to value 0.
	 * @param x the x-coordinate
	 * @param y the y-coordinate
	 * @param width width of the changed rectangle. 
	 * @param height height of the changed rectangle.
	 * @param model the WallModel that changed
	 */
	public void ledsChanged(int x, int y, int width, int height, WallModel model);
}
