/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.server;

import no.ntnu.idi.wall.model.DefaultWallModel;
import no.ntnu.idi.wall.model.WallModel;

/**
 * An implementation of Protocol that executes the Protocol operations on a WallModel, e.g. clearWall clears the model. 
 * @author  Hallvard Trtteberg
 */
public class ProtocolModel implements Protocol {

	/**
	 * @uml.property  name="model"
	 * @uml.associationEnd  
	 */
	private WallModel model;

	/**
	 * This instance operates on the gives WallModel
	 * @param model
	 */
	public ProtocolModel(WallModel model) {
		this.model = model;
	}
	/**
	 * This instance operates on a fresh DefaultWallModel instance.
	 */
	public ProtocolModel() {
		this(new DefaultWallModel());
	}
	
	public void clearWall() {
		fillRect((short)0, (short)0, (short)WallModel.WIDTH, (short)WallModel.HEIGHT, (short)0);
	}

	private void setLed(final int x, final int y, final int value) {
		model.setLed(x, y, (byte)value);
	}
	
	public void fillRect(short x, short y, short w, short h, short value) {
		model.setLeds(x, y, w, h, (byte)value);
	}

	public void getLeds(short x, short y, short w, short h, short[] ip, int port) {
		// TODO
	}

	public void nop() {
	}

	public void setLed(short x, short y, short value) {
		setLed((int)x, (int)y, (int)value);
	}

	public void setLeds(short x, short y, short w, short h, short[] value, int offset) {
		int count = 0;
		for (int dy = 0; dy < h; dy++) {
			for (int dx = 0; dx < w; dx++) {
				setLed(x + dx, y + dy, value[offset + count++]);
			}
		}
	}

	public void sync() {
		if (model instanceof DefaultWallModel) {
			((DefaultWallModel)model).notifyListeners();
		}
	}
}
