/*
 * 
 * Copyright (c) 2008, Hallvar Trætteberg All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of the Norwegian University of Technology and Science nor
 *   the names of its contributors may be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
package no.ntnu.idi.wall.ui;

import java.awt.Color;
import java.awt.Graphics;

import no.ntnu.idi.wall.model.WallModel;

public class StandardLedRenderer implements LedRenderer {

	private Color altOffColor = new Color(6, 6, 6);
	private Color offColor = Color.BLACK;

	private Color[] colors = new Color[256];
	
	private Color getLedColor(int value, boolean altColor) {
		int colorNum = value * (altColor ? 2 : 1);
		if (colors[colorNum] == null) {
			Color color = (altColor ? altOffColor : offColor);
			if (value != 0) {
				int red = (255 - color.getRed()) * value / WallModel.MAX_LED_VALUE + color.getRed();
				color = new Color(red, color.getGreen(), color.getBlue());
			}
			colors[colorNum] = color;
		}
		return colors[colorNum];
	}
	
	public void paintLed(int x, int y, int value, Graphics g, int ledX, int ledY, int w, int h) {
		boolean altColor = (x / WallModel.NODE_WIDTH) % 2 == (y / WallModel.NODE_HEIGHT) % 2;
		g.setColor(getLedColor(0, altColor));
		g.fillRect(ledX, ledY, w, h);
		g.setColor(getLedColor(value, altColor));
		g.fillOval(ledX + w / 3, ledY + h / 3, w / 3, h / 3);
	}
}
