/*
 * Copyright 2009 UbiCollab.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ubicollab.service.openwall.ui;


import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ercp.swt.mobile.*;
import org.ubicollab.service.openwall.model.Controller;
import org.ubicollab.service.openwall.model.IOWPresenter;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import org.eclipse.swt.SWT;


/**
 * @author    Simone
 * @uml.dependency   supplier="org.ubicollab.service.openwall.ui.WallDrawer"
 */
public class MainView extends ViewPart implements PaintListener, KeyListener, SelectionListener{
	
    Canvas  canvas;
	int     drawCase=8;
	int     counter;
	boolean eraseBgd;
	Command commandExit;
	Command commandNext;
    private FocusListener superShellListener = null;
	Color col1;
	Color col2;
	public static final String ID = "MainView"; //$NON-NLS-1$
	/**
	 * @uml.property  name="controller"
	 * @uml.associationEnd  
	 */
	Controller controller=null;
	/**
	 * @uml.property  name="wall"
	 * @uml.associationEnd  
	 */
	WallDrawer wall = new WallDrawer();
	
	public MainView() {	
	}

	/**
	 * Create contents of the view part.
	 */
	public void createPartControl(Composite parent) {
		controller=Controller.getInstance();
		controller.registerView(this);
		
		//CONNECTION TO THE WALL
		wall.connect();
		
		
		//Fill the static reference to the Display
		Controller.setDisplay(parent.getDisplay());
		 // In this showcase, we have to create and use a shell instead of the composite because,
	    // if we draw in the composite, the screen is flickering because the Top Level Shell does not
	    // have the NO_BACKGROUND flag. We cannot set it, because it can cause problems in the other
	    // showcases. Consequently, we are using a child Shell with this flag enabled. 
	    canvas = new Canvas(parent, SWT.NO_BACKGROUND);
	    
	    //Splash message boxs
	    /*
	    MessageBox messageBox = new MessageBox(parent.getShell(),SWT.ICON_INFORMATION);
		messageBox.setText("About");
		messageBox.setMessage("Hello, this is an application for the Ubicollab Platform");
		messageBox.open();
		MessageBox messageBox2 = new MessageBox(parent.getShell(),SWT.ICON_INFORMATION);
		messageBox.setText("How to Use");
		messageBox.setMessage("Tap on the screen to browse the photos");
		messageBox.open();
		*/
	    
		//Listener su canvas
	    canvas.addMouseListener(new MouseListener() {
	    	public void mouseDoubleClick(MouseEvent arg0) {
	    	}
	    	public void mouseDown(MouseEvent arg0) {
	    	}
	    	public void mouseUp(MouseEvent arg0) {
					drawNext();
	    	}
	    });
	        
	    //Define the screen painting area
	    Rectangle area = parent.getClientArea();
	    canvas.setBounds(area);
	    canvas.addPaintListener(this);
	    canvas.addKeyListener(this);	
		eraseBgd = true;

		// create Commands			
		commandNext = new Command(canvas, Command.SELECT,3);
		commandNext.setText("command NEXT"); //$NON-NLS-1$
		commandNext.addSelectionListener(this);
		commandNext.setDefaultCommand();

		commandExit = new Command(canvas, Command.EXIT,1);
		commandExit.setText("label"); //$NON-NLS-1$
		commandExit.setLongLabel("longlabel"); //$NON-NLS-1$
		commandExit.addSelectionListener(this);
		
	    canvas.setFocus();
	    
		//Bring up this cavas image if existed while clicking eSWT Demo shortcut.
		superShellListener = new FocusListener() {
			public void focusGained(FocusEvent arg0) {
				if(canvas != null && !canvas.isDisposed()) {
					++drawCase;
					eraseBgd = true;
					//Trigger a redraw event
					canvas.redraw();
					canvas.forceFocus();
				}
			}

			public void focusLost(FocusEvent arg0) {
			}			
		};
	}

	 //WS METHODS
    public void drawNext()
    {
    	++drawCase;
		eraseBgd = true;
		canvas.redraw();
		wall.updateWall(drawCase);
    }
    public void drawPrevious()
    {
    	--drawCase;
		eraseBgd = true;
		canvas.redraw();
    }
    public void drawTarget(int index)
    {
    	drawCase = index;
    	eraseBgd = true;
		canvas.redraw();
		wall.updateWall(drawCase);
    }
    
	/**
	 * Drawing operations
	 */
	//When a redraw event happen, it choose the next photo and draw it
	public void paintControl(PaintEvent e) {
		if (eraseBgd) {
			e.gc.fillRectangle(canvas.getClientArea());
			eraseBgd = false;
		}
		switch (drawCase) {
			case 0:	    
				//drawImage(e, "/res/1.jpg"); //$NON-NLS-1$
				break;
			case 1:	    
				drawImage(e, "/res/1.jpg"); //$NON-NLS-1$
				break;
			case 2:
				drawImage(e, "/res/2.jpg"); //$NON-NLS-1$
				break;
			case 3:
				drawImage(e, "/res/3.jpg"); //$NON-NLS-1$
				break;
			case 4:	    
				drawImage(e, "/res/4.jpg"); //$NON-NLS-1$
				drawCase=0;
				break;
			case 8:	    
				drawImage(e, "/res/8.jpg"); //$NON-NLS-1$
				drawCase=0;
				break;
			default:
				quit();
				break;
		}
	}
	
	/**
	 * Press any key to go through the different screens
	 */
	public void keyPressed(KeyEvent e) {
	/** 
	 * Marked out temporarily due to the cause of duplicating 'canvas.redraw()' action when hitting 
	 * Enter/'1' keys for Command Selection Event.
	 * */
		// ++drawCase;
		// eraseBgd = true;
		// canvas.redraw();
	}
	
	public void keyReleased(KeyEvent e) {
	}
	
	/**
	 * display an image 
	 * 
	 * @param e	Paintevent
	 * @param name filename
	 */
	private void drawImage(PaintEvent e, String name) {
		Image img = new Image(canvas.getDisplay(), getClass().getResourceAsStream(name));
		Rectangle imageBnds  = img.getBounds();
		Rectangle canvasBnds = canvas.getBounds();
		int x = (canvasBnds.width  - imageBnds.width ) / 2;
		int y = (canvasBnds.height - imageBnds.height) / 2;
		GC gc = new GC(canvas, SWT.NONE);
		gc.fillRectangle(canvas.getClientArea());
		gc.drawImage(img, x, y);
		img.dispose();
		/*
		String url = "http://folk.ntnu.no/simonem/bundles/bundle_support/img.jpg";
		URL uri = null;
		try {
			uri = new URL(url);
		} catch (MalformedURLException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}
		try {
			Image img2 = new Image(canvas.getDisplay(),uri.openStream());
			gc.drawImage(img2, x, y);
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}*/
		
		
	}
		
	public void widgetSelected(SelectionEvent e) {
		if (e.widget==commandExit) {
		    counter = 0;
			quit();
		}
		else if (e.widget==commandNext) {
			++drawCase;
			eraseBgd = true;
			canvas.redraw();
		}
	}
	
	public void quit() {
	    if (col1 != null) {
	        col1.dispose();
	    }
	    if (col2 != null) {
	        col2.dispose();
	    }
	    if(canvas.getShell() != null && !canvas.getShell().isDisposed())
		{	
	    	canvas.getParent().getShell().removeFocusListener(superShellListener);
		}   
	    canvas.getParent().redraw();
	    dispose();
	}
	public void widgetDefaultSelected(SelectionEvent e) {
 	}
	public void setFocus() {
		// set the focus
	}
}
