/*
 * Copyright 2009 UbiCollab.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ubicollab.service.openwall.ui;

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ImageProducer;
import java.awt.image.ReplicateScaleFilter;

import java.net.UnknownHostException;

import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import no.ntnu.idi.wall.image.RGB2LedImageFilter;
import no.ntnu.idi.wall.image.WallImageFilter;
import no.ntnu.idi.wall.image.WallModelImageConsumer;
import no.ntnu.idi.wall.model.DefaultWallModel;
import no.ntnu.idi.wall.model.WallModel;
import no.ntnu.idi.wall.model.WallModelProtocolClient;


/**
 * @author  Simone
 */
public class WallDrawer {

	/**
	 * @uml.property  name="model"
	 * @uml.associationEnd  
	 */
	private DefaultWallModel model = new DefaultWallModel();
	/**
	 * @uml.property  name="protocolClient"
	 * @uml.associationEnd  
	 */
	private WallModelProtocolClient protocolClient = null; 


	public WallDrawer() {
		
	}
	
	public BufferedImage produceBufferedImage(int target)
	{
		File file = new File("c:/res/"+target+".jpg");
		BufferedImage img = null;
		try {
			img = ImageIO.read(file);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("FILE NOT FOUND");
		}
		return img;
		
	}

	public ImageProducer preview(int target) {
		ImageProducer source = null;
		BufferedImage image = produceBufferedImage(target); // convertComponent2Image(imageView);
		source = createImageSource(image);
		return source;
	}
	
	public ImageProducer createImageSource(Image image) {
		ImageFilter[] filters = new ImageFilter[] {
				new ReplicateScaleFilter(WallModel.WIDTH, WallModel.HEIGHT),
	//			new CropImageFilter(
	//					(image.getWidth() - WallModel.WIDTH) / 2,
	//					(image.getHeight() - WallModel.HEIGHT) / 2,
	//					WallModel.WIDTH, WallModel.HEIGHT),
						new RGB2LedImageFilter(255, 0, 0),
		};
		ImageProducer source = image.getSource();
		for (int i = 0; i < filters.length; i++) {
			source =  new FilteredImageSource(source, filters[i]);
		}
		return source;
	}
	

	
	public void updateWall(int target) {
		ImageProducer source = preview(target);
		model.clearLeds();
		if (source != null) {
			source.startProduction(new WallModelImageConsumer(model));
		}
		if (protocolClient != null) {
			protocolClient.updateAll(model);
		}
	}


	public void connect() {
		String serverHost = "wallidi.idi.ntnu.no";
		try {
			protocolClient = new WallModelProtocolClient(serverHost, null, 0, 0);
		} catch (UnknownHostException uhe) {
			System.err.println(uhe);
			serverHost = serverHost + " (unknown)"; 
		}
	}

	
}
